<?php
/**
 * Zend Frameworkを利用したRSS/Atom Reader『ZFReader』
 * ルーター
 * 
 * @author ishinao <ishinao@ishinao.net>
 * $Id: Router.php 72 2006-05-15 09:19:25Z ishinao $
 */

/**
 * Router関連クラスを読み込み
 */
Zend::loadInterface('Zend_Controller_Router_Interface');
Zend::loadClass('Zend_Controller_Dispatcher_Token');

/**
 * ルータークラス
 * 
 * Zend Frameworkの規約では、Zend Framework関連クラスのサブクラスを
 * 作る場合、[プロジェクト名]ディレクトリ下にZend Frameworkと同様の
 * クラス規約に基づいたカスタムクラスを配置することになっている。
 */
class ZFReader_Controller_Router implements Zend_Controller_Router_Interface
{
	/**
	 * ルーティング設定を配列で定義しておく。各項目は、
	 * 
	 * '[パターンマッチ]' => array(
	 *    array([マッチインデックス] => [パラメータ名], ...),
	 *    array([デフォルトパラメータ名] => [デフォルト値]), 
	 * );
	 * 
	 * という内容。たとえば、
	 * 
	 * '/^read(\/(\d+))?$/' => array(
	 *    array(2 => 'id'),
	 *    array('action' => 'read'),
	 * )
	 * 
	 * という定義は、/^read(\/(\d+))?$/というパターンにマッチした場合、
	 * アクション名は'read'となり、マッチした2番目の部分はidパラメータとなる。
	 * 「/read/123」だったら、idは123となる。
	 * 
	 * パラメータ名のうち「action」と「controller」は予約されており、
	 * DsipacherTokenのactionとcontrollerとして解釈される。
	 * 
	 * $routesを順番にチャックしていき、最初にマッチしたパターンがあった段階で
	 * そのルーティング設定が有効になる。
	 */
	protected $_routes = array(
		// ログアウト
		'/^logout$/' => array(
			array(),
			array('controller' => 'system', 'action' => 'logout'),
		),
		// フィードを読む
		'/^read(\/(\d+))?$/' => array(
			array(2 => 'id'),
			array('action' => 'read'),
		),
		// フィードを追加
		'/^add$/' => array(
			array(),
			array('action' => 'add'),
		),
		// フィード情報の編集
		'/^edit\/(\d+)$/' => array(
			array(1 => 'id'),
			array('action' => 'edit'),
		),
		// フィードの削除
		'/^remove\/(\d+)$/' => array(
			array(1 => 'id'),
			array('action' => 'remove'),
		),
		// すべてのフィードアイテム情報の更新
		'/^crawl$/' => array(
			array(),
			array('action' => 'crawlAll'),
		),
		// 指定したフィードアイテム情報の更新
		'/^crawl\/(\d+)?$/' => array(
			array(1 => 'id'),
			array('action' => 'crawl'),
		),
		// AJAX用フィード情報および新着アイテム情報の取得
		'/^api\/feed\/(\d+)$/' => array(
			array(1 => 'id'),
			array('action' => 'feedInfo'),
		),
		// トップ
		'/^$/' => array(
			array(),
			array('action' => 'index')
		),
		// その他
		'//' => array(
			array(),
			array(
				'controller' => 'system',
				'action' => 'error',
				'message' => '無効なリクエストです'
			)
		),
		
	);

	/**
	 * routeメソッド
	 */
    public function route(Zend_Controller_Dispatcher_Interface $dispatcher)
    {

    	$server = Zend::registry('server'); // $_SERVER入力フィルタ
    	$pathinfo = trim($server->getRaw('PATH_INFO'), '/'); // $_SERVER['PATH_INFO']

		$controllerName = 'index'; // コントローラクラス名
		$actionName = 'index'; // アクションメソッド名
		$params = array(); // アクションパラメータ
		
		// ルーティング設定のチェック
		foreach ($this->_routes as $pattern => $config) {
			
			if (preg_match($pattern, $pathinfo, $matches)) { // パターンにマッチした
				
				$paramConfig = $config[0]; // マッチパラメータ設定
				$defaultConfig = $config[1]; // デフォルトパラメータ設定
				
				if (isset($defaultConfig['controller'])) {

					// 'controller'パラメータはコントローラ名に
					$controllerName = $defaultConfig['controller'];
					unset($defaultConfig['controller']);
					
				}
				
				if (isset($defaultConfig['action'])) {
					
					// 'action'パラメータはアクション名に
					$actionName = $defaultConfig['action'];
					unset($defaultConfig['action']);
					
				}
				
				$params = $defaultConfig; // デフォルトパラメータをセット
				
				// マッチしたパラメータをセット
				foreach ($paramConfig as $index => $name) {
					$params[$name] = $matches[$index];
				}
				
				break;
			}
		}
		
		// DispatcherTokenを生成
    	$token = new Zend_Controller_Dispatcher_Token(
    		$controllerName,
    		$actionName,
    		$params 
    	);

        if (!$dispatcher->isDispatchable($token)) {

			return new Zend_Controller_Dipatcher_Token(
				'system',
				'error',
				array('message' => '無効なリクエストです')
			);
        	
        }

        return $token; // Tokenを返す

	}
	
	/**
	 * PATH_INFO指定からサイト内のリンクURLを解決する
	 * 
	 * PATH_INFOを使ってURLを解決しているので、サイト内のルートURLは、
	 * 「/」ではなく、「/path/to/ZFReader/index.php/」になる。
	 * そこで、リンク先やリダイレクト先の解決に、PATH_INFOから
	 * URLに解決するためのメソッドを用意しておき、これを使って変換をかける
	 */
	public static function getUrl($path)
	{

		$server = Zend::registry('server'); // $_SERVER入力フィルタ
		return $server->getRaw('SCRIPT_NAME') . $path;  // ルートからの絶対URLに変換して返す

	}
}

?>
